/* -----------------------------------------------------------------------------
*
*  npmysql.h
*
*  ANTz - realtime 3D data visualization tools for the real-world, based on NPE.
*
*  ANTz is hosted at http://openantz.com and NPE at http://neuralphysics.org
*
*  Written in 2010-2013 by Shane Saxon - contact@saxondigital.net
*
*  Please see main.c for a complete list of additional code contributors.
*
*  To the extent possible under law, the author(s) have dedicated all copyright 
*  and related and neighboring rights to this software to the public domain
*  worldwide. This software is distributed without any warranty.
*
*  Released under the CC0 license, which is GPL compatible.
*
*  You should have received a copy of the CC0 Public Domain Dedication along
*  with this software (license file named COPYING.txt). If not, see
*  http://creativecommons.org/publicdomain/zero/1.0/
*
* --------------------------------------------------------------------------- */

#ifndef NPMYSQL_H_
#define NPMYSQL_H_

//#include <mysql.h>
#include "../../npdata.h"
#include "../gl/nptags.h" // Maybe something to relocate
//#include "../../data/npmapfile.h"

#include <my_global.h>
#include <mysql.h>
#include "mysqlStructures.h"
#include "mysqlInitFuncs.h"
#include "mysqlSetFuncs.h"
#include "mysqlGetFuncs.h"  // Maybe this should be in mysqlLoadIntoAntz
#include "mysqlGenFuncs.h"
#include "mysqlFormatFuncs.h"
#include "mysqlLoadIntoAntz.h"


enum
{
	MySQL = 0,
	Oracle = 1
};

enum {
	Create = 0,
	Insert = 1,
	Select = 2,
	Use = 3,
	Drop = 4,
	Alter = 5,
	Show = 6
};

enum {
	NODE_STATE_TABLE = 0,
	TAGS_TABLE = 1,
	CHMAP_TABLE = 2,
	CHANNELS_TABLE = 3
};

//------------------------------------------------------------------------------ 
void npInitAllChunks(struct chunksObj *chunks, void* dataRef);

struct chunksObj* npNewerAllChunk(struct csvNodes *nodes, void* dataRef);

char*** npNewFilterIDbyInterval(struct dbConnectMysql *connect, char ***buffer, int interval, void* dataRef);

struct databaseObject* npInitDbDatabase(struct dbNewConnect *connect, char *name, void* dataRef);

int npdbGenFields(struct dbNewConnect *connect, char* fields, int type, void* dataRef);

void npOpenDbConnect(struct dbNewConnect *connection, void* dataRef);

void npNewInitDbConnectMySQL(struct dbNewConnect *connection, char* host, char* user, char* pass, char* db);

int npDropDbTable(struct dbNewConnect *connect, char *name, void* dataRef);

int npSetDbQueryValues(struct dbNewConnect *connect, struct databaseObject *object, struct chunksObj *valueChunks);

struct databaseObject* npdbUseDatabase(struct dbNewConnect *connect, char *name, void* dataRef);

int npAttachMySQLtoDataRef(struct dbNewConnect *connect, void* dataRef);

int npdbExecuteQuery(struct dbNewConnect *connect, struct databaseObject *dbQuery, void* dataRef);

int npdbGenQuery(struct dbNewConnect *connect, struct databaseObject *dbQuery, void* dataRef);

void npOpenDbConnectMySQL(struct dbNewConnect *connection, void* dataRef);

void npTest(void* dataRef);

int npSetDbQueryCommand(struct dbNewConnect *connect, struct databaseObject *dbQuery, int command, void* dataRef);

char** npNewGetRetrieveTextTagsValues(void* dataRef);

struct csvNodes* npNewRetrieveChMapValues(void* dataRef);

void npInitDbConnection(struct dbNewConnect *connection, int dbType, char* host, char* user, char* pass, char* db);

char*** npGetRootNodesIds(struct dbNewConnect *connect, char*** buffer, int interval, void* dataRef);

void npOpenMySQL (struct dbConnectMysql *connection, void* dataRef);

void npNewExecuteMysqlQuery(struct dbConnectMysql *connect,struct mysqlObject *object, void* dataRef);


void npExecuteQueryBufferMySQL(struct dbConnectMysql *connect, char* buffer, struct dbConnect *connection, int *ErrResult, void* dataRef);

struct chunksObj* npRetrieve(struct dbConnectMysql *connect, int valueType, void* dataRef);

void npCloseMySQL (struct dbConnectMysql *connect, void* dataRef);

void npUpdateMySQL (struct dbConnectMysql *connect, void* dataRef);

void npSetDbQueryFields(struct dbNewConnect *connect, struct databaseObject *object, char *fields);

void* npGetResultStructureFromConnectionMysql(struct dbConnectMysql *connect, void* dataRef);

int npFreeMysqlQuery(struct dbConnectMysql *connect, struct mysqlObject *object, void* dataRef);

int npFreeMysqlObject(struct dbConnectMysql *connect, struct mysqlObject *object, void* dataRef);

void npInitDbConnectMySQL (struct dbConnectMysql *connection, char* host, char* user, char* pass, char* db);

char** npGetNodesValues2(void *dataRef);

void npNewMapTraverseTreeLL(struct csvNodes *nodes, int* index, pNPnode node, int format, void* dataRef);

struct csvNodes* npFixedNodeValues(void *dataRef);

char** npGetNodesValues3(int arbritary, void *dataRef);

char*** npChunk(char **buffer, int arbritary, void *dataRef);

int npFreeMysqlDatabase(struct dbConnectMysql *connect, struct mysqlObject *object, void* dataRef);

int npFreeMysqlTable(struct dbConnectMysql *connect, struct mysqlObject *object, void* dataRef);

int npCreateAntzDatabaseMysql(struct dbConnectMysql *connect, void* dataRef);

int npCreateDatabaseMysql(struct dbConnectMysql *connect, char *name, void* dataRef);

int	npUseAntzDatabaseMysql(struct dbConnectMysql *connect, void* dataRef);

int npUseAntzDatabase(struct dbNewConnect *connect, void* dataRef);

int npUseDatabaseMysql(struct dbConnectMysql *connect, char *name, void* dataRef);

int	npCreateAntzNodeStateTable(struct dbNewConnect *connect, void* dataRef);

int npCreateTableMysql(struct dbConnectMysql *connect, char *name, char *fields, void* dataRef);

int npCreateAntzTagsTable(struct dbNewConnect *connect, void* dataRef);

int npInsertAntzTags(struct dbNewConnect *connect, void* dataRef);

int npInsertIntoTableMysql(struct dbConnectMysql *connect, char *name, char ***values, int numChunks, void* dataRef);

int npInsertAntzNodeStates(struct dbConnect *connect, void* dataRef);

int npDropAntzTagsTable(struct dbNewConnect *connect, void* dataRef);

int npSelectAntzTagsTable(struct dbConnect *connect, void* dataRef);

int npSelectTableMysql(struct dbConnectMysql *connect, struct mysqlObject *query, char *name, void* dataRef);

int npAntzMysqlCtrl(struct dbConnect *connect, int command, int table, void* dataRef);

void npLoadMysqlResultIntoAntzMysql(struct dbConnectMysql *connect, struct mysqlObject *object, int tableType, void* dataRef);

int npDropAntzTableMysql(struct dbConnectMysql *connect, int tableType, void* dataRef);

int npInsertAntzTableMysql(struct dbConnectMysql *connect, int tableType, void* dataRef);

int npCreateAntzTableMysql(struct dbConnectMysql *connect, int tableType, void* dataRef);

int npCreateAntzNodeStateTableMysql(struct dbConnectMysql *connect, void* dataRef);

char*** npNewChunk(char **buffer, int arbritary, void *dataRef);

void npNewTest(void* dataRef);

struct databaseObject* npdbStoreResult(struct dbNewConnect *connect, struct databaseObject *dbQuery, void* dataRef);

int npAntzDatabaseMysql(struct dbConnectMysql *connect, int command, void* dataRef);

int npAntzMysql(struct dbConnectMysql *connect, int objectType, int tableType, int command, void* dataRef);

int npSelectAntzTableMysql(struct dbConnectMysql *connect, int tableType, void* dataRef);

struct databaseObject* npInitDbResult(struct dbNewConnect *connect, void* dataRef);

struct databaseObject* npdbCtrl(struct dbNewConnect *connect, int objectType, int tableType, char *name, char *fields, struct chunksObj *valueChunks, int conditionalType, char* conditionals, int command, void* dataRef);

int npLoadAntzStateFromDatabase(struct dbNewConnect *connect, void* dataRef);

int npSaveAntzStateToDatabaseMysql(struct dbConnectMysql *connect, void* dataRef);

int npInsertIntoDbTable(struct dbNewConnect *connect, char *name, char ***values, int numChunks, void* dataRef);

struct databaseObject* npInitDbQuery(struct dbNewConnect *connect);

struct databaseObject* npInitDbTable(struct dbNewConnect *connect, char* name);

int npSetDbQueryObject(struct dbNewConnect *connect, struct databaseObject *dbQuery, struct databaseObject *dbTable, void* dataRef);

int npNewInsertIntoTable(struct dbNewConnect *connect, char *name, char **values, int numChunks, void* dataRef);

//int npSaveAntzStateToDatabase(struct dbConnect *connect, void* dataRef);
int npSaveAntzStateToDatabase(struct dbNewConnect *connect, void* dataRef);

int npGetNodeRangeFromMysql(struct dbConnectMysql *connect, char* field, char* from, char* to, void* dataRef);

int npSetMysqlQueryConditionals(struct dbConnectMysql *connect, struct mysqlObject *query, int conditionalType, char* conditional, void* dataRef);

int npGetRootNodesFromMysql(struct dbConnectMysql *connect, void* dataRef);

int npGetNodesWithMysql(struct dbConnectMysql *connect, int fieldType, int fieldValue, void* dataRef);

char** npRetrieveChMapValues(char** buffer, int *numberOfChunks, void* dataRef);

int npCreateAntzChMapTable(struct dbNewConnect *connect, void* dataRef);

int npSelectAntzChMapTable(struct dbConnect *connect, void* dataRef);

int npDropAntzChMapTable(struct dbConnect *connect, void* dataRef);

int npSelectAntzChannelsTable(struct dbConnect *connect, void* dataRef);

int npInsertAntzChannelsTable(struct dbConnect *connect, void* dataRef);

char** npRetrieveChannelsValues(struct dbConnect *connect, char** buffer, void* dataRef);

//int npSampleNodeStateTable(struct dbConnect *connect, int mod, void* dataRef);
int npSampleNodeStateTable(struct dbNewConnect *connect, int mod, void* dataRef);

char*** npGetResultMysql(struct dbConnectMysql *connect, MYSQL_RES *result, char*** buffer, void* dataRef);

//char*** npGetRootNodesIdsFromMysql(struct dbConnect *connect, char*** buffer, int interval);

char*** npGetRootNodesIdsFromMysql(struct dbConnectMysql *connect, char*** buffer, int interval, void* dataRef); // Add buffer

char*** npGetChildNodeFromRecordIdMysql(struct dbConnectMysql *connect, char* recordID, void* dataRef);

char*** npFilterIDbyInterval(struct dbConnectMysql *connect, char ***buffer, int interval, void* dataRef);

//int npLoadRootNodes(char*** buffer, void* dataRef);
int npLoadRootNodes(struct dbConnectMysql *connect, char*** buffer, void* dataRef);


int npGetChildren(struct dbConnectMysql *connect, char*** buffer, void* dataRef);


#endif

